# IBM_PROLOG_BEGIN_TAG
#
# serviceable.event.provider 1.0.0
#
# Licensed Materials - Property of IBM
#
# Restricted Materials of IBM
#
# COPYRIGHT International Business Machines Corp. 2014,2014
# All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# IBM_PROLOG_END_TAG
'''
Created on Apr 17, 2014
@author: Sowjanya Rao
'''
import time
import os
import subprocess
import datetime
import SnmpConstants
from datetime import date
from LogEntry import LogEntry
from ParseErrorLog import ParseErrorLog
from ConfigParser import ConfigParser
from SnmpTrapUtils import SnmpTrapUtils

class MonitorErrorLog:
    def tail(self, filename):
        try:
            if os.path.exists(filename) :
                popen = subprocess.Popen('tail -n 1000 -F '+filename, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True);
                while True:
                    line = popen.stdout.readline().strip()
                    if line:
                        yield line
        except:
            self.tail(filename)
      
    def process_matches(self, matchtext, filecreatedtime_withoutmillis):
     
        while True:
            line = (yield)
            try:
                if matchtext.lower() in line.lower():
                    parseErrorLog = ParseErrorLog()
                    logEntry = parseErrorLog.parseLogEntry(line)
    #               email alert, etc.
                                       
                    if SnmpConstants.alertQueue.qsize() <= SnmpConstants.MAX_QUEUE_SIZE :
                        loggeddate=logEntry.date+" "+str(date.today().year)
                        loggedtime=logEntry.time
                        string_date=loggeddate+" "+loggedtime
                        logentrytime = datetime.datetime.strptime(string_date, "%b %d %Y %H:%M:%S")
                        if (filecreatedtime_withoutmillis <= logentrytime) :
                            SnmpConstants.alertQueue.put(logEntry)
                            if not os.path.exists(SnmpConstants.LAST_ERROR_FILE_PATH) :
                                os.makedirs(SnmpConstants.LAST_ERROR_FILE_PATH)
                            open(SnmpConstants.LAST_ERROR_FILE, 'w')
                        else :
                            SnmpTrapUtils().tracelogger().info("Entry ignored")
                    else :
                        SnmpTrapUtils().tracelogger().error("Queue reached Maximum limit:"+str(SnmpConstants.MAX_QUEUE_SIZE));
            except:
                SnmpTrapUtils().tracelogger().error("Error occurred while processing the line: "+str(line)+" "+sys.exc_info()[0]);
     
    def monitor(self):
        config = ConfigParser()
        config.read(SnmpConstants.CONFIGFILE)
        syslog = config.get("SYS_LOG", "SYS_LOG")
        if os.path.exists(SnmpConstants.LAST_ERROR_FILE):
            ftimestamp = os.path.getmtime(SnmpConstants.LAST_ERROR_FILE)
            filecreatedtime = datetime.datetime.fromtimestamp(ftimestamp)                    
        else : 
            filecreatedtime = datetime.datetime.now()  
        filecreatedtime_withoutmillis = filecreatedtime.replace(microsecond=0)
        if os.path.isfile(syslog) == False:
            syslog = SnmpConstants.SYS_LOG
        list_of_matches = [SnmpConstants.REQ_SERVICE_ACTION_AND_CALL_HOME,SnmpConstants.REQ_SERVICE_ACTION_AND_NO_CALL_HOME]
        matches = [self.process_matches(string_match,filecreatedtime_withoutmillis) for string_match in list_of_matches]
        for m in matches:  # prime matches
            m.next()
    
        while True:
            auditlog = self.tail(syslog)
            for line in auditlog:
                for m in matches:
                    m.send(line)
