# IBM_PROLOG_BEGIN_TAG
#
# serviceable.event.provider 1.0.0
#
# Licensed Materials - Property of IBM
#
# Restricted Materials of IBM
#
# COPYRIGHT International Business Machines Corp. 2014,2014
# All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# IBM_PROLOG_END_TAG
'''
Created on Apr 17, 2014
@author: Sowjanya Rao
'''
from operator import attrgetter
from datetime import datetime
from collections import defaultdict
import re
import sys
from LogEntry import LogEntry
import logging
from SnmpTrapUtils import SnmpTrapUtils

class ParseErrorLog:
    
    def parseLogEntry(self, line):
        regex = "(?P<date>^\s*[\w]+\s*[\d]+)\s*(?P<time>[\d:]+)\s*(?P<host>[\w\W]+)\s*(ELOG|DIAG_ENCL)['['](?P<elog>[\d\w]+)]:\s*[L][I][D]['['](?P<lid>[a-zA-Z0-9]+)]::[S][R][C]['['](?P<src>[\w\W]+)]::(?P<failure>[\w\W]+)::(?P<errorType>[\w\W]+)::(?P<action>[\w\W]+)"
        SnmpTrapUtils().tracelogger().info('New Log entry detected. Parsing - started')
        compiledEntry = None
        logentry = None
        
        try:
            compiledEntry = re.compile(regex)
        except:
            SnmpTrapUtils().tracelogger().error('Error while compiling the regex')
        
        try:
             parsedEntry = compiledEntry.search(line)
        except:
            SnmpTrapUtils().tracelogger().error('Error while parsing the log entry')
            
        if parsedEntry != None :    
            date = parsedEntry.group('date')
            time = parsedEntry.group('time')
            host = parsedEntry.group('host')
            elog = parsedEntry.group('elog')
            lid = parsedEntry.group('lid')
            src = parsedEntry.group('src')
            failure = parsedEntry.group('failure')
            errorType = parsedEntry.group('errorType')
            action = parsedEntry.group('action')
            logentry = LogEntry(date, time, host, elog, lid, src, failure, errorType, action)
        
            infoString = 'date=' + str(date) + ' time=' + str(time) + 'host=' + str(host) + ' se=' + str(elog) + ' lid=' + str(lid) + ' src=' + str(src) + ' failure=' + str(failure) + ' errorType=' + str(errorType) + ' action=' + str(action)
            SnmpTrapUtils().tracelogger().info(infoString)
        else:
            SnmpTrapUtils().tracelogger().info("log entry is null")
            
        return logentry
       


    
   
