# IBM_PROLOG_BEGIN_TAG
#
# serviceable.event.provider 1.0.0
#
# Licensed Materials - Property of IBM
#
# Restricted Materials of IBM
#
# COPYRIGHT International Business Machines Corp. 2014,2014
# All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# IBM_PROLOG_END_TAG
'''
Created on Apr 17, 2014
@author: Chen Shao Jie
'''
import time
import SnmpTrapEntry
import SnmpConstants
import threading
import SnmpSendTrap
import os
import logging.config

class SnmpTrapUtils:
    
    def tracelogger(self):
        logging.config.fileConfig(SnmpConstants.LOG4P_PATH)
        _tracelogger = logging.getLogger("trace")
        return _tracelogger
    
    def activitylogger(self):
        logging.config.fileConfig(SnmpConstants.LOG4P_PATH)
        _activitylogger = logging.getLogger("activity")
        return _activitylogger
    
    def getSystemInfo(self, logEntry):
        if len(self._sysvalue) > 0:
            self._sysType = self._sysvalue[3]
            self._sysModel = self._sysvalue[1]
            self._sysSerialNumber = self._sysvalue[2]
            self._sysFirmware = self._sysvalue[8]
            self._VMID = self._sysvalue[6]
            self._sysOSName = self._sysvalue[5]
            self._sysOSType = self._sysvalue[5]
            self._sysOSVersion = self._sysvalue[4]
            self._agentHostName = self._sysvalue[0]
            self._agentHostIP = self._sysvalue[7]
        
        self._agentName = SnmpConstants.VERSION
        self._agentType = ""
        self._trapAppID = "PowerKVMSEprovider"
        self._trapDateTime = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(time.time()))
        if logEntry.action.strip() == SnmpConstants.REQ_SERVICE_ACTION_AND_CALL_HOME.replace(":", "").strip():
            self._actionFlg = "2"
        else:
            self._actionFlg = "1"
            
        self._extendedErrorData = ""
        self._addtionalDetails = ""
        self._seTestFlg = SnmpConstants.TEST_CONTENT
        
    def setSystemInfo(self, logEntry):
        try:
            self._fo = open(SnmpConstants.SYS_INFO, "r")
            line = self._fo.readline()
            if line:
                # hostname:model:serial:type:OSVersion:OSName:LPAR:hostip:firmware
                self._sysvalue = line.strip().split("@@")
        except:
            # add log
            SnmpTrapUtils().activityLog("UNKNOW HOST", False, logEntry)
            SnmpTrapUtils().tracelogger().error("SnmpTrapUtils::SystemInfo open system info file error")
            raise Exception
        finally:
            self._fo.close()
     
    def activityLog(self, host, flg=True, logEntry=None):
        msg = logEntry.date + str(" ") + logEntry.time + str(" ") + logEntry.host \
        + str(" SE[") + logEntry.elog + str("]:LID[") + logEntry.lid + str("]::SRC[") \
        + logEntry.src + str("]::") + logEntry.failure + str("::") + logEntry.errorType +str("::")+logEntry.action \
        + str("::alert transmission to <") + host
        if flg:
            msg += str("> is successful")
            SnmpTrapUtils().activitylogger().info(msg)
        else:
            msg += str("> failed due to: <failure while sending trap>")
            SnmpTrapUtils().activitylogger().error(msg)
        
    def getTrapInfo(self, logEntry):
        SnmpTrapUtils.setSystemInfo(self, logEntry)
        SnmpTrapUtils.getSystemInfo(self, logEntry)
        snmpTrapEntry = SnmpTrapEntry.SnmpTrapEntry(self._trapDateTime, self._VMID, self._trapAppID, self._agentName, self._agentType,
                                      self._agentHostIP, self._agentHostName, self._sysType , self._sysModel, self._sysSerialNumber,
                                      self._sysFirmware, self._sysOSVersion, self._sysOSName, self._sysOSType, logEntry.date + str(" ") + logEntry.time, logEntry.lid,
                                      logEntry.src, logEntry.failure, self._actionFlg, self._extendedErrorData, self._addtionalDetails, self._seTestFlg)
        return snmpTrapEntry
        
        
    def convertStr(self, string):
        return " s '" + string + "'"
    
    def setParameters(self, dest, trapOid, snmpTrapEntry, community):
        commandSendTrap = SnmpConstants.CMD_PATH + str(" -v 2c -c ")
        commandSendTrap += community + " " + dest + " '' "
        commandSendTrap += trapOid + " "
        commandSendTrap += SnmpConstants.SE_TRAP_DATETIME + SnmpTrapUtils.convertStr(self, snmpTrapEntry.seTrapDateTime) + " "
        commandSendTrap += SnmpConstants.SE_ERROR_OCCURTIME + SnmpTrapUtils.convertStr(self, snmpTrapEntry.seErrorOccurTime) + " "
        commandSendTrap += SnmpConstants.SE_LOGID + SnmpTrapUtils.convertStr(self, snmpTrapEntry.seLogId) + " "
        commandSendTrap += SnmpConstants.SE_SRC + SnmpTrapUtils.convertStr(self, snmpTrapEntry.seSRC) + " "
        commandSendTrap += SnmpConstants.SE_FAILIING_SUB_SYS + SnmpTrapUtils.convertStr(self, snmpTrapEntry.seFailingSubSys) + " "
        commandSendTrap += SnmpConstants.SE_ACTION_FLG + SnmpTrapUtils.convertStr(self, snmpTrapEntry.seActionFlg) + " "
        commandSendTrap += SnmpConstants.SE_TRAP_APP_ID + SnmpTrapUtils.convertStr(self, snmpTrapEntry.seTrapAppID) + " "
        commandSendTrap += SnmpConstants.SE_AGENT_NAME + SnmpTrapUtils.convertStr(self, snmpTrapEntry.seAgentName) + " "
        commandSendTrap += SnmpConstants.SE_AGENT_TYPE + SnmpTrapUtils.convertStr(self, snmpTrapEntry.seAgentType) + " "
        commandSendTrap += SnmpConstants.SE_AGENT_HOSTIP + SnmpTrapUtils.convertStr(self, snmpTrapEntry.seAgentHostIP) + " "
        commandSendTrap += SnmpConstants.SE_AGENT_HOSTNAME + SnmpTrapUtils.convertStr(self, snmpTrapEntry.seAgentHostName) + " "
        commandSendTrap += SnmpConstants.SE_SYS_OS_NAME + SnmpTrapUtils.convertStr(self, snmpTrapEntry.seSysOSName) + " "
        commandSendTrap += SnmpConstants.SE_SYS_OS_TYPE + SnmpTrapUtils.convertStr(self, snmpTrapEntry.seSysOSType) + " "
        commandSendTrap += SnmpConstants.SE_SYS_OS_VERSION + SnmpTrapUtils.convertStr(self, snmpTrapEntry.seSaOSVersion) + " "
        commandSendTrap += SnmpConstants.SE_SYS_TYPE + SnmpTrapUtils.convertStr(self, snmpTrapEntry.seSysType) + " "
        commandSendTrap += SnmpConstants.SE_SYS_MODEL + SnmpTrapUtils.convertStr(self, snmpTrapEntry.seSysModel) + " "
        commandSendTrap += SnmpConstants.SE_SYS_SERIAL_NUMBER + SnmpTrapUtils.convertStr(self, snmpTrapEntry.seSysSerialNumber) + " "
        commandSendTrap += SnmpConstants.SE_SYS_FIRMWARE + SnmpTrapUtils.convertStr(self, snmpTrapEntry.seSysFirmware) + " "
        commandSendTrap += SnmpConstants.SE_VMID + SnmpTrapUtils.convertStr(self, snmpTrapEntry.seVMID) + " "
        commandSendTrap += SnmpConstants.SE_EXTENDED_ERRORDATA + SnmpTrapUtils.convertStr(self, snmpTrapEntry.seExtendedErrorData) + " "
        commandSendTrap += SnmpConstants.SE_ADDITIONAL_DETAILS + SnmpTrapUtils.convertStr(self, snmpTrapEntry.seAdditionalDetails) + " "
        if trapOid == SnmpConstants.IBM_TEST_TRAP_SERVICE:
            commandSendTrap += SnmpConstants.SE_TEST_FLG + SnmpTrapUtils.convertStr(self, str(snmpTrapEntry.seTestFlg))
        SnmpConstants.CMD_SEND_TRAP = commandSendTrap.strip()
        
    def startSysDiscover(self):
        # collect system information before starting sysDiscover
        try:
            os.system(SnmpConstants.CMD_GERNERATE_SYS_INFO)
            SnmpTrapUtils().tracelogger().info("System info was collected")
        except:
            SnmpTrapUtils().tracelogger().error("SnmpTrapUtils::startSysDiscover collection error")
            
        _modifyTime = os.path.getmtime(SnmpConstants.SYS_INFO)
        while True:
            time.sleep(SnmpConstants.INTERVAL)
            if _modifyTime != os.path.getmtime(SnmpConstants.SYS_INFO):
                try:
                    _modifyTime = os.path.getmtime(SnmpConstants.SYS_INFO)
                    os.system(SnmpConstants.CMD_GERNERATE_SYS_INFO)
                    SnmpTrapUtils().tracelogger().info("System info was collected")
                except:
                    SnmpTrapUtils().tracelogger().error("SnmpTrapUtils::startSysDiscover collection error")
                
    def startSendTrap(self):
        # waiting for system information collection before starting sending traps
        time.sleep(10)
        SnmpTrapUtils().tracelogger().info("Ready for sending traps...")
        while True:
            if SnmpConstants.stopSendTrap:
                break
            # remove unAlive threads
            if len(SnmpConstants.threadsNumber) <= SnmpConstants.MAX_THREAD:
                for _thread in SnmpConstants.threadsNumber:
                    if _thread.isAlive() == False:
                        SnmpConstants.threadsNumber.remove(_thread)
                         
            if SnmpConstants.alertQueue.empty() == False:
                if len(SnmpConstants.threadsNumber) == 0:
                    # start a new tread, and save thread ID
                    thread = _sendTrapTread()
                    thread.start()
                    SnmpConstants.threadsNumber.append(thread)
                else:
                    # check theread id list, if list.size()<maxthreadNaumber, then start a new thread
                    if len(SnmpConstants.threadsNumber) < SnmpConstants.MAX_THREAD:
                        thread = _sendTrapTread()
                        thread.start()
                        SnmpConstants.threadsNumber.append(thread)
                
    def stopSendTrap(self):
        # kill threads
        for _thread in SnmpConstants.threadsNumber:
            _thread.__stop()
        SnmpConstants.stopSendTrap = True
        return
    
class _sendTrapTread(threading.Thread):
    def __init__(self):
        self._logEntry = SnmpConstants.alertQueue.get()
        threading.Thread.__init__(self)
    
    def run(self):
        self._snmpSendTrap = SnmpSendTrap.SnmpSendTrap(self._logEntry)
        self._snmpSendTrap.sendTrap()
        
        